Package: rblimp
Type: Package
Title: Integration of 'Blimp' Software into R
Version: 1.2.0
Authors@R: 
    person(given = "Brian T.",
           family = "Keller",
           role = c("aut", "cre", "cph"),
           email = "btkeller@missouri.edu")
Description: 
    Provides an interface to 'Blimp' software for Bayesian latent variable modeling,
    missing data analysis, and multiple imputation. The package generates 'Blimp' syntax,
    executes 'Blimp' models, and imports results back into 'R' as structured objects with
    methods for visualization and analysis. Requires 'Blimp' software (freely available
    at <https://www.appliedmissingdata.com/blimp>) to be installed separately.
License: GPL-3
Encoding: UTF-8
Depends: R (>= 4.4.0)
SystemRequirements: Blimp software
        (<https://www.appliedmissingdata.com/blimp>)
Imports: cli, methods, ggplot2
Suggests: mitml, rstudioapi, posterior
RoxygenNote: 7.3.3
URL: https://github.com/blimp-stats/rblimp
BugReports: https://github.com/blimp-stats/rblimp/issues
NeedsCompilation: no
Packaged: 2026-05-18 01:13:51 UTC; Bry
Author: Brian T. Keller [aut, cre, cph]
Maintainer: Brian T. Keller <btkeller@missouri.edu>
Repository: CRAN
Date/Publication: 2026-05-18 07:00:02 UTC
Built: R 4.6.0; ; 2026-05-18 10:04:23 UTC; unix
