# Generated by roxygen2: do not edit by hand

S3method("[",blimp_out)
S3method(as.character,blimp_syntax)
S3method(posterior::as_draws,blimp_obj)
S3method(posterior::as_draws_array,blimp_obj)
S3method(posterior::as_draws_df,blimp_obj)
S3method(print,blimp_out)
S3method(print,blimp_syntax)
export(SIMULATE)
export(algorithm)
export(as.mitml)
export(by_group)
export(compare)
export(compute_condeff)
export(datainfo)
export(describe)
export(detect_blimp)
export(estimates)
export(has_blimp)
export(install_blimp)
export(jn_plot)
export(jn_plot_func)
export(model_table)
export(modelfit)
export(modelinfo)
export(output)
export(posterior_plot)
export(psr)
export(rblimp)
export(rblimp_fcs)
export(rblimp_sim)
export(rblimp_source)
export(rblimp_syntax)
export(residual_plot)
export(set_blimp)
export(simple_plot)
export(standardized)
export(trace_plot)
export(uninstall_blimp)
export(update_blimp)
export(write.blimp)
exportClasses(blimp_cp)
exportClasses(blimp_obj)
exportClasses(describe)
exportMethods(as.array)
exportMethods(as.data.frame)
exportMethods(as.matrix)
exportMethods(describe)
exportMethods(names)
exportMethods(predict)
exportMethods(resid)
exportMethods(residuals)
exportMethods(show)
exportMethods(summary)
exportMethods(with)
exportMethods(write.blimp)
import(ggplot2)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert)
importFrom(cli,cli_alert_warning)
importFrom(ggplot2,as_labeller)
importFrom(grDevices,xy.coords)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,dotchart)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,points)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(stats,coef)
importFrom(stats,loess)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,ppoints)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,resid)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,start)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(utils,combn)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,write.csv)
