% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blimp_obj.R
\name{residuals,blimp_obj-method}
\alias{residuals,blimp_obj-method}
\title{Residuals scores from \code{blimp_obj}}
\usage{
\S4method{residuals}{blimp_obj}(object, ...)
}
\arguments{
\item{object}{A \code{blimp_obj} object}

\item{...}{Additional arguments (unused)}
}
\value{
A list of data frames, one per imputation, each containing the
residual columns from the model.
}
\description{
Residuals scores from \code{blimp_obj}
}
